#!/usr/bin/env python
#
# This file is part of REPA: Release Engineering Process Assistant.
#
# Copyright (C) 2013 Intel Corporation
#
# REPA is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""
REPA: Release Engineering Process Assistant.

Copyright (C) Intel Corporation 2013
Licence: GPL version 2
Author: Ed Bartosh <eduard.bartosh@intel.com>

Common module.
Common functions, classes, exceptions.
"""

import json

from repa.obs import OBS_PREFIX

class RepaException(Exception):
    """Custom repa exception. All repa modules should use it."""
    pass

def get_project_by_name(obs, name):
    """Lookup for a project in OBS by submission or group name."""
    projects = list(obs.get_projects('^%s.*:%s$' % (OBS_PREFIX,
                                                    name.replace('/', ':'))))
    if not projects:
        raise RepaException('OBS project not found for %s' % name)
    if len(projects) > 1:
        raise RepaException('%s resolves into multiple projects' % name)

    return projects[0][0], json.loads(projects[0][1])


def _resolve_submissions(obs, name):
    """Get list of submissions with meta. Resolves submitgroups."""
    project, meta = get_project_by_name(obs, name)
    if name.startswith('submitgroup'):
        for subm in meta['submissions']:
            sprj, smeta = get_project_by_name(obs, subm)
            yield subm, sprj, smeta
    else:
        yield name, project, meta


def delete_project(obs, name):
    """Delete OBS project related to submission or submit group."""
    project = get_project_by_name(obs, name)[0]
    obs.delete_project(project)


def accept_or_reject(obs, submission, state, comment=''):
    """Create SRs and set their state for one submission or for a group."""
    for name, project, meta in _resolve_submissions(obs, submission):
        # osc submitreq [OPTIONS] SOURCEPRJ SOURCEPKG DESTPRJ [DESTPKG]
        # osc request accept [-m TEXT] ID
        print "submission %s" % str(name)

        message = "Comments: %s \nGit project: %s\nTag: %s\nCommit: %s" \
            % (comment or "submission %s" % str(name),
              meta['projects'][0],
              meta['git_tag'],
              meta['git_tag'])

        for pkg in obs.get_source_packages(project):
            # Create SR
            reqid = obs.create_sr(project, pkg, str(meta['obs_target_prj']),
                                  message=str(message))
            print 'package %s: created SR %s' % (pkg, reqid)

            # and immediately set its state
            message = "SR %s is set to %s" % (reqid, state)
            if comment:
                message += comment
            obs.set_sr_state(reqid, state=state,
                             message=str(message), force=True)
            print 'set SR state to', state

    # delete submit group
    if submission.startswith('submitgroup'):
        delete_project(obs, submission)

