#!/usr/bin/env python
#
# This file is part of REPA: Release Engineering Process Assistant.
#
# Copyright (C) 2013 Intel Corporation
#
# REPA is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""
REPA: Release Engineering Process Assistant.

Copyright (C) Intel Corporation 2013
Licence: GPL version 2
Author: Ed Bartosh <eduard.bartosh@intel.com>

OBS module.
Interface module to access Open Build System.

This module is a intermediate step for new APIs
to oscapi. As soon as APIs are mature enough they
should be contributed to oscapi.
"""

import sys
import re

from base64 import b64encode
from xml.etree import cElementTree as ET
from StringIO import StringIO

from osc import core

from gitbuildsys.oscapi import OSC, ObsError, OSCError
from gitbuildsys.utils import Temp


OBS_PREFIX = "home:prerelease:"

OSCRC_TEMPLATE = """[general]
apiurl = %(apiurl)s
plaintext_passwd=0
use_keyring=0
http_debug = %(http_debug)s
debug = %(debug)s
gnome_keyring=0
[%(apiurl)s]
user=%(user)s
passx=%(passwdx)s
"""

class OBS(OSC):
    """Interface to OBS API."""

    def __init__(self, apiurl, apiuser, apipasswd):
        oscrc = OSCRC_TEMPLATE % {
            'http_debug': 0,
            "debug": 0,
            "apiurl": apiurl,
            "user": apiuser,
            "passwdx": b64encode(apipasswd.encode('bz2'))}

        self.apiurl = apiurl
        tmpf = Temp(prefix='.oscrc', content=oscrc)
        self.oscrcpath = tmpf.path
        OSC.__init__(self, apiurl, self.oscrcpath)


    def get_projects(self, regexp=''):
        """List projects with attributes."""
        for project in core.meta_get_project_list(self.apiurl):
            if regexp and not re.match(regexp, project):
                continue
            yield project, self.get_description(project)


    def get_build_results(self, prj):
        """Get project build results."""
        meta = core.show_prj_results_meta(self.apiurl, prj)
        root = ET.fromstring(''.join(meta))
        buildres = {}
        if not root.find('result'):
            return buildres
        for results in root.findall('result'):
            key = (results.get('repository'), results.get('arch'))
            buildres[key] = dict([(field, results.get(field)) \
                                      for field in ('code', 'state')])
            pkgresults = []
            for node in results:
                code = node.get('code')
                if node.get('code') != 'excluded':
                    pkgresults.append((node.get('package'), code))
            buildres[key]['packages'] = pkgresults

        return buildres


    def get_source_packages(self, prj):
        """Get list of binary packages in the project."""
        return iter(core.meta_get_packagelist(self.apiurl, prj))


    def get_binary_packages(self, prj):
        """Get list of binary packages in the project."""
        for repo in core.get_repos_of_project(self.apiurl, prj):
            yield (repo.name, repo.arch), core.get_binarylist(self.apiurl,
                                                              prj,
                                                              repo.name,
                                                              repo.arch)

    @staticmethod
    def aggregate_package(src_project, src_package, dst_project,
                          dst_package):
        """Aggregate package. Wraps core.aggregate_pack."""
        saved = sys.stdout
        sys.stdout = StringIO()
        try:
            core.aggregate_pac(src_project, src_package,
                               dst_project, dst_package)
        finally:
            sys.stdout = saved


    def create_sr(self, src_project, package, dst_project,
                  message='', revision=None):
        """Create submit request for the project."""
        rev = revision
        if not rev:
            url = core.makeurl(self.apiurl, ['source', src_project, package],
                               query="expand=1")
            try:
                answer = self.core_http(core.http_GET, url)
            except OSCError, err:
                raise ObsError("cat't query source %s from project %s: %s" % \
                               (package, src_project, err))
            rev = ET.parse(answer).getroot().get('rev')
        return core.create_submit_request(self.apiurl, src_project, package,
                                          dst_project, package, message=message,
                                          orev=rev)

    def set_sr_state(self, reqid, state, message='', force=False):
        """Set SR state."""
        return core.change_request_state(self.apiurl, reqid, state,
                                         message=message, force=force)
