#!/usr/bin/env python
#
# This file is part of REPA: Release Engineering Process Assistant.
#
# Copyright (C) 2013 Intel Corporation
#
# REPA is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

"""
REPA: Release Engineering Process Assistant.

Copyright (C) Intel Corporation 2013
Licence: GPL version 2
Author: Ed Bartosh <eduard.bartosh@intel.com>

List module.
Get list of submissions.
"""

import sys
import json

from repa.obs import OBS, OBS_PREFIX
from repa.main import sub_main


def get_status(obs, project):
    """Gest submission status."""
    # Process project build results
    codes = set()
    for target in obs.get_build_results(project).itervalues():
        codes.add(target.get('code'))
        codes.add(target.get('state'))
        for pkginfo in target['packages']:
            codes.add(pkginfo[1])
    status = ','.join(codes) or 'None'
    return status


def show_images(meta):
    if 'images' in meta:
        print '    images:', ' '.join("%s:%s" % (img['name'], img['status']) \
                                      for img in meta['images'])

def list_submissions(obs, regexp):
    """List submissions and groups."""
    # submissions
    groups = []
    for project, desc in obs.get_projects('^%s.*%s' % (OBS_PREFIX, regexp)):
        meta = json.loads(desc)
        if ':submitgroup:' in project:
            groups.append(meta)
            continue
        print meta['git_tag'], meta['obs_target_prj'],
        print '    projects: ', ' '.join(meta['projects'])
        print "    builds:", get_status(obs, project)
        show_images(meta)
        print

    # groups
    for meta in groups:
        print meta['name']
        print '    submissions:', ' '.join(subm.split('/')[-1] \
                                           for subm in meta['submissions'])
        show_images(meta)
        print


class List(object):
    """Subcommand: List submissions."""

    name = 'list'
    description = 'List submissions'
    help = description

    @staticmethod
    def add_arguments(parser, config):
        """
        Add arguments to the parser. Called from [sub_]main.
        Set defaults for arguments from config.
        """
        parser.add_argument('-r', '--regexp', help='search regexp',
                            default=config.get('list_regexp', '.*'))

    @staticmethod
    def run(argv):
        """Command line entry point. Called from [sub_]main"""
        obs = OBS(argv.apiurl, argv.apiuser, argv.apipasswd)
        return list_submissions(obs, argv.regexp)


if __name__ == '__main__':
    sys.exit(sub_main(sys.argv[1:], List()))
